Open bus
--------
The NES and Famicom connect different input wires to the CPU's data
bus when input ports $4016 and $4017 are read.  The NES connects bits
0, 3, and 4 of both ports to the 7-pin connector, while the Famicom
connects bit 0 of both ports to the hardwired controllers, bit 4 of
$4016 to the second controller's microphone (counterintuitively), and
bit 1 of $4016 and bits 1-4 of $4017 to the DA15 expansion connector.
Any bits not connected to something produce an effect called "open
bus", where capacitance holds the previous value on the data bus in
place.  So we can detect which system we're on through its open bus
behavior.

To test for open bus, we have to arrange for particular values to be
on the data bus before reading the input ports.  Normally, the last
thing on the data bus when the CPU reads memory is the last byte of
the instruction, which is the upper byte of the address.  This limits
which bits we can control somewhat.

But we can use another quirk of the 6502 to control more bits.
Indexed addressing modes first try performing a partial 8-bit
addition, adding the low byte of the index register (X or Y) to the
low byte of the register modulo 256 and using the unchanged high byte
in the address that the CPU reads.  Oftentimes, the 8-bit addition
does not produce a carry (a value larger than 255), and reading this
partial result is correct.  But when this 8-bit addition produces a
carry, it has to add 1 to the high byte and perform the read again.
Any side effects of having performed the first read have already
taken effect.  One important side effect is that whatever is on the
data bus from the first read will carry over to the second.

There are eight important addresses for the open bus test.

* $3F06 and $3F16  
  All addresses from $2000 through $3FFF are mapped to the PPU, and
  the PPU sees only the low 3 bits.  This means $3F06 and $3F16
  behave the same as $2006.  However, $2006 is the write-only video
  memory address port, and reads produce open bus.  For example,
  reading $3F from the instruction and then reading one of these
  addresses will leave $3F on the data bus.
* $3F07 and $3F17  
  These behave as $2007, the video memory data port.  This port is
  readable, and we can arrange for reads of both $00 and $FF values.
* $4006 and $4007  
  These are write-only ports associated with the second pulse wave
  tone generator.  Reading results in open bus: $40 from an absolute
  read or whatever the PPU left on the bus for an indexed read with
  a carry.
* $4016 and $4017
  These are the input ports.  Some bits are driven; others are open
  bus.  Bits 7-5 in particular are open bus on any system other than
  the coin-operated Vs. System.

Before any controller detection happens, it checks that the hardware
quirks on which open bus detection relies are present.  Some flash
carts and emulators are known to interfere with these quirks.
First it makes sure the CPU can read back $00 and $FF values in
the nametable through the PPU.  This is done through $2007 and its
mirrors $3F07 and $3F17.  Then to confirm that the cartridge is not
interfering with the open bus, it looks for the address high byte in
absolute reads from $2006, $3F06, $3F07, $4006, and $4007, as well as
a $3F byte from a $3F06-$4006 sequence.  Finally it looks for video
memory readback with a $3F07-$4007 sequence.  If all these pass, the
machine handles open bus well enough to test all bits of $4017 and
the low 7 bits of $4016.

This produces 16 bytes of results:
* $2007: 00 FF
* $3F07: 00 FF
* $3F17: 00 FF
* $3F07-$4007: 00 FF
* $4006: 00
* $4007: 00
* $2000: 20
* $2001: 20
* $2006: 20
* $3F06: 3F
* $3F16: 3F
* $3F06-$4006: 3F

If a machine doesn't emulate enough open bus behavior to allow a test
to proceed, the NES vs. Famicom part displays a diagnostic.

* 001: PPU readback  
  The PPU doesn't read back data from video memory through $2007.
  A machine like this will not boot _Super Mario Bros._
* 002: PPU mirror readback  
  The PPU doesn't read back data from video memory through
  $3F07 and $3F17.
* 003: APU instruction open bus  
  Reads from $4006 and $4007 don't produce $40.  A machine like this
  will not read button presses in Mindscape games.  At one point, the
  PowerPak had this problem because it used pull resistors on the
  data bus.
* 004: PPU instruction open bus  
  Reads from $2006, $3F06, and $3F16 don't produce $20 or $3F as
  appropriate.
* 005: Double open bus  
  Consecutive reads from $3F06 and $4006 don't produce $3F.
* 006: PPU readback open bus  
  The PPU doesn't read back data from video memory through a
  $3F07-$4007 sequence.

Detecting controllers
---------------------
Several controllers are "parallel", meaning they produce separate
bits on separate wires.  Others, such as the standard gamepad, are
"serial", meaning the bits change on successive reads to indicate
separate input states.  Some are both, such as the Arkanoid
controller that has a button on one wire and a serial potentiometer
reading on the other.  Serial streams will end with a long string of
usually 1 bits, occasionally 0 bits.

So it reads each controller twice, taking two streams of 32 bits each
time, and separate the bits into four classes: bits that stay 1 the
whole time, bits that stay 0 the whole time, bits that vary within
a stream (probably serial), and bits that come from open bus.  The
first controller ($4016) is read once with absolute open bus ($40)
and once with an indexed read ($3F).  The second controller ($4017)
is read with PPU readback as the indexed open bus source: once with
$40 and once with VRAM filled with $3F.

The reading routine produces eight values:

* Minimum (bitwise AND) of 32 reads of $4016 with $40 open bus
* Minimum (bitwise AND) of 32 reads of $4016 with $3F open bus
* Maximum (bitwise OR) of $4016 with $40 open bus
* Maximum (bitwise OR) of $4016 with $3F open bus
* Minimum (bitwise AND) of 32 reads of $4017 with $40 open bus
* Minimum (bitwise AND) of 32 reads of $4017 with $3F open bus
* Maximum (bitwise OR) of $4017 with $40 open bus
* Maximum (bitwise OR) of $4017 with $3F open bus

These feed into four rules of thumb for bits 6-0 of $4016 and $4017:

* If the minimum and maximum with $40 open bus differ, that line
  is serial.
* If the minimum and maximum with all four open bus are all 0, that
  line is 0.
* If the minimum and maximum with all four open bus are all 1, that
  line is 1.
* If the minimum and maximum with $40 open bus are the same, and the
  minimum and maximum with $3F open bus are the same, that line is
  open bus.
